@@ -7,11 +7,12 @@ import traceback
 from pathlib import Path
 
 import matplotlib
+import matplotlib as mpl
 import numpy as np
 from PySide6 import QtCore, QtGui, QtQuick, QtWidgets
 from PySide6.QtCore import Qt, qInstallMessageHandler, QMessageLogContext, QtMsgType
 
-from matplotlib import cbook
+from matplotlib import cbook, _api
 from matplotlib.backend_bases import FigureCanvasBase, NavigationToolbar2, MouseButton, TimerBase
 from matplotlib.backend_tools import cursors
 from matplotlib.backends.backend_agg import FigureCanvasAgg
@@ -748,6 +749,200 @@ class FigureCanvasQtQuickAgg(FigureCanvasAgg, FigureCanvasQtQuick):
         super().print_figure(*args, **kwargs)
         self.draw()
 
+
+class NavigationToolbar2QT(NavigationToolbar2, QtWidgets.QToolBar):
+    message = QtCore.Signal(str)
+
+    toolitems = [*NavigationToolbar2.toolitems]
+
+    #TODO too many stuf be involve here
+    # toolitems.insert(
+    #     # Add 'customize' action after 'subplots'
+    #     [name for name, *_ in toolitems].index("Subplots") + 1,
+    #     ("Customize", "Edit axis, curve and image parameters",
+    #      "qt4_editor_options", "edit_parameters"))
+
+    def __init__(self, canvas, parent, coordinates=True):
+        """coordinates: should we show the coordinates on the right?"""
+        QtWidgets.QToolBar.__init__(self, parent)
+        self.setAllowedAreas(
+            QtCore.Qt.ToolBarArea_Mask.TopToolBarArea
+            | QtCore.Qt.ToolBarArea_Mask.TopToolBarArea)
+
+        self.coordinates = coordinates
+        self._actions = {}  # mapping of toolitem method names to QActions.
+
+        for text, tooltip_text, image_file, callback in self.toolitems:
+            if text is None:
+                self.addSeparator()
+            else:
+                a = self.addAction(self._icon(image_file + '.png'),
+                                   text, getattr(self, callback))
+                self._actions[callback] = a
+                if callback in ['zoom', 'pan']:
+                    a.setCheckable(True)
+                if tooltip_text is not None:
+                    a.setToolTip(tooltip_text)
+
+        # Add the (x, y) location widget at the right side of the toolbar
+        # The stretch factor is 1 which means any resizing of the toolbar
+        # will resize this label instead of the buttons.
+        if self.coordinates:
+            self.locLabel = QtWidgets.QLabel("", self)
+            self.locLabel.setAlignment(
+                QtCore.Qt.AlignmentFlag.AlignRight
+                | QtCore.Qt.AlignmentFlag.AlignVCenter)
+            self.locLabel.setSizePolicy(QtWidgets.QSizePolicy(
+                QtWidgets.QSizePolicy.Expanding,
+                QtWidgets.QSizePolicy.Ignored,
+            ))
+            labelAction = self.addWidget(self.locLabel)
+            labelAction.setVisible(True)
+
+        NavigationToolbar2.__init__(self, canvas)
+
+    @_api.deprecated("3.3", alternative="self.canvas.parent()")
+    @property
+    def parent(self):
+        return self.canvas.parent()
+
+    @_api.deprecated("3.3", alternative="self.canvas.setParent()")
+    @parent.setter
+    def parent(self, value):
+        pass
+
+    @_api.deprecated(
+        "3.3", alternative="os.path.join(mpl.get_data_path(), 'images')")
+    @property
+    def basedir(self):
+        return str(cbook._get_data_path('images'))
+
+    def _icon(self, name):
+        """
+        Construct a `.QIcon` from an image file *name*, including the extension
+        and relative to Matplotlib's "images" data directory.
+        """
+        if QtCore.qVersion() >= '5.':
+            name = name.replace('.png', '_large.png')
+        pm = QtGui.QPixmap(str(cbook._get_data_path('images', name)))
+        _setDevicePixelRatio(pm, _devicePixelRatioF(self))
+        if self.palette().color(self.backgroundRole()).value() < 128:
+            icon_color = self.palette().color(self.foregroundRole())
+            mask = pm.createMaskFromColor(
+                QtGui.QColor('black'),
+                QtCore.Qt.MaskMode.MaskOutColor)
+            pm.fill(icon_color)
+            pm.setMask(mask)
+        return QtGui.QIcon(pm)
+
+    #TODO fix in the future
+    # def edit_parameters(self):
+    #     axes = self.canvas.figure.get_axes()
+    #     if not axes:
+    #         QtWidgets.QMessageBox.warning(
+    #             self.canvas.parent(), "Error", "There are no axes to edit.")
+    #         return
+    #     elif len(axes) == 1:
+    #         ax, = axes
+    #     else:
+    #         titles = [
+    #             ax.get_label() or
+    #             ax.get_title() or
+    #             " - ".join(filter(None, [ax.get_xlabel(), ax.get_ylabel()])) or
+    #             f"<anonymous {type(ax).__name__}>"
+    #             for ax in axes]
+    #         duplicate_titles = [
+    #             title for title in titles if titles.count(title) > 1]
+    #         for i, ax in enumerate(axes):
+    #             if titles[i] in duplicate_titles:
+    #                 titles[i] += f" (id: {id(ax):#x})"  # Deduplicate titles.
+    #         item, ok = QtWidgets.QInputDialog.getItem(
+    #             self.canvas.parent(),
+    #             'Customize', 'Select axes:', titles, 0, False)
+    #         if not ok:
+    #             return
+    #         ax = axes[titles.index(item)]
+    #     figureoptions.figure_edit(ax, self)
+
+    def _update_buttons_checked(self):
+        # sync button checkstates to match active mode
+        if 'pan' in self._actions:
+            self._actions['pan'].setChecked(self.mode.name == 'PAN')
+        if 'zoom' in self._actions:
+            self._actions['zoom'].setChecked(self.mode.name == 'ZOOM')
+
+    def pan(self, *args):
+        super().pan(*args)
+        self._update_buttons_checked()
+
+    def zoom(self, *args):
+        super().zoom(*args)
+        self._update_buttons_checked()
+
+    def set_message(self, s):
+        self.message.emit(s)
+        if self.coordinates:
+            self.locLabel.setText(s)
+
+    def set_cursor(self, cursor):
+        self.canvas.setCursor(cursord[cursor])
+
+    def draw_rubberband(self, event, x0, y0, x1, y1):
+        height = self.canvas.figure.bbox.height
+        y1 = height - y1
+        y0 = height - y0
+        rect = [int(val) for val in (x0, y0, x1 - x0, y1 - y0)]
+        self.canvas.drawRectangle(rect)
+
+    def remove_rubberband(self):
+        self.canvas.drawRectangle(None)
+
+    #TODO fix in the future
+    # def configure_subplots(self):
+    #     image = str(cbook._get_data_path('images/matplotlib.png'))
+    #     dia = SubplotToolQt(self.canvas.figure, self.canvas.parent())
+    #     dia.setWindowIcon(QtGui.QIcon(image))
+    #     qt_compat._exec(dia)
+
+    def save_figure(self, *args):
+        filetypes = self.canvas.get_supported_filetypes_grouped()
+        sorted_filetypes = sorted(filetypes.items())
+        default_filetype = self.canvas.get_default_filetype()
+
+        startpath = os.path.expanduser(mpl.rcParams['savefig.directory'])
+        start = os.path.join(startpath, self.canvas.get_default_filename())
+        filters = []
+        selectedFilter = None
+        for name, exts in sorted_filetypes:
+            exts_list = " ".join(['*.%s' % ext for ext in exts])
+            filter = '%s (%s)' % (name, exts_list)
+            if default_filetype in exts:
+                selectedFilter = filter
+            filters.append(filter)
+        filters = ';;'.join(filters)
+
+        fname, filter = _getSaveFileName(
+            self.canvas.parent(), "Choose a filename to save to", start,
+            filters, selectedFilter)
+        if fname:
+            # Save dir for next time, unless empty str (i.e., use cwd).
+            if startpath != "":
+                mpl.rcParams['savefig.directory'] = os.path.dirname(fname)
+            try:
+                self.canvas.figure.savefig(fname)
+            except Exception as e:
+                QtWidgets.QMessageBox.critical(
+                    self, "Error saving file", str(e),
+                    QtWidgets.QMessageBox.Ok, QtWidgets.QMessageBox.NoButton)
+
+    def set_history_buttons(self):
+        can_backward = self._nav_stack._pos > 0
+        can_forward = self._nav_stack._pos < len(self._nav_stack._elements) - 1
+        if 'back' in self._actions:
+            self._actions['back'].setEnabled(can_backward)
+        if 'forward' in self._actions:
+            self._actions['forward'].setEnabled(can_forward)
+
 #TODO may crash sometime
 class FigureCanvasQT(QtWidgets.QWidget, FigureCanvasBase):
     required_interactive_framework = "qt"
